import { Op } from 'sequelize';
import { AppDemoModel } from './models/index.ts';
import { app } from '@/app.ts';

app
  .route({
    path: 'app-demo',
    key: 'list',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { page = 1, pageSize = 20, search, sort = 'DESC' } = ctx.query;
    const searchWhere = search
      ? {
          [Op.or]: [{ title: { [Op.like]: `%${search}%` } }, { summary: { [Op.like]: `%${search}%` } }],
        }
      : {};

    const { rows: appDemo, count } = await AppDemoModel.findAndCountAll({
      where: {
        uid: tokenUser.uid,
        ...searchWhere,
      },
      offset: (page - 1) * pageSize,
      limit: pageSize,
      order: [['updatedAt', sort]],
    });

    ctx.body = {
      list: appDemo,
      pagination: {
        page,
        current: page,
        pageSize,
        total: count,
      },
    };
  })
  .addTo(app);

app
  .route({
    path: 'app-demo',
    key: 'update',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { id, data, updatedAt: _clear, createdAt: _clear2, ...rest } = ctx.query.data;
    let appDemo: AppDemoModel;
    let isNew = false;
    if (id) {
      const appDemo = await AppDemoModel.findByPk(id);
      if (appDemo.uid !== tokenUser.uid) {
        ctx.throw(403, 'No permission');
      }
    } else {
      appDemo = await AppDemoModel.create({
        data: data,
        ...rest,
        uid: tokenUser.uid,
      });
      isNew = true;
    }
    if (!appDemo) {
      ctx.throw(404, 'AppDemo not found');
    }
    if (!isNew) {
      appDemo = await appDemo.update({
        data: { ...appDemo.data, ...data },
        ...rest,
      });
    }

    ctx.body = appDemo;
  })
  .addTo(app);

app
  .route({
    path: 'app-demo',
    key: 'delete',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { id, force = false } = ctx.query.data || {};
    if (!id) {
      ctx.throw(400, 'id is required');
    }
    const appDemo = await AppDemoModel.findByPk(id);
    if (appDemo.uid !== tokenUser.uid) {
      ctx.throw(403, 'No permission');
    }
    await appDemo.destroy({ force });
    ctx.body = appDemo;
  })
  .addTo(app);

app
  .route({
    path: 'app-demo',
    key: 'get',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { id } = ctx.query.data || {};
    if (!id) {
      ctx.throw(400, 'id is required');
    }
    const appDemo = await AppDemoModel.findByPk(id);
    if (appDemo.uid !== tokenUser.uid) {
      ctx.throw(403, 'No permission');
    }
    ctx.body = appDemo;
  })
  .addTo(app);
